use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Brohman
CPAN
Chase
David
Etheridge
Fredric
Golden
Ivan
Karen
Kent
Moore
Olaf
Randy
Shawn
Sibley
Spelling
Stauner
Test
Thomas
Tubert
Whitener
brian
capoeirab
dagolden
ether
foy
github
itub
kentfredric
lib
olaf
preprocessing
randy
sartak
stopwords
trs
wordlist
